@@ -20,10 +20,12 @@ def _find_library_candidates(library_names,
     """
     Finds and returns filenames which might be the library you are looking for.
     """
-    candidates = set()
-    for library_name in library_names:
-        for search_path in library_search_paths:
-            glob_query = os.path.join(search_path, '*'+library_name+'*')
+    candidates = []
+    for search_path in library_search_paths:
+        if not search_path:
+            continue
+        for library_name in library_names:
+            glob_query = os.path.join(search_path, '*'+library_name+'.*')
             for filename in glob.iglob(glob_query):
                 filename = os.path.realpath(filename)
                 if filename in candidates:
@@ -38,11 +40,11 @@ def _find_library_candidates(library_names,
                 for file_extension in library_file_extensions:
                     if basename_end.startswith(file_extension):
                         if basename_end[len(file_extension):][:1] in ('', '.'):
-                            candidates.add(filename)
-                    if basename_end.endswith(file_extension):
+                            candidates.append(filename)
+                    elif basename_end.endswith(file_extension):
                         basename_middle = basename_end[:-len(file_extension)]
                         if all(c in '0123456789.' for c in basename_middle):
-                            candidates.add(filename)
+                            candidates.append(filename)
     return candidates
 
 
@@ -55,7 +57,7 @@ def _load_library(library_names, library_file_extensions,
                                           library_file_extensions,
                                           library_search_paths)
     library_versions = []
-    for filename in candidates:
+    for filename in set(candidates):
         version = version_check_callback(filename)
         if version is not None and version >= (3, 0, 0):
             library_versions.append((version, filename))
@@ -66,6 +68,39 @@ def _load_library(library_names, library_file_extensions,
     return ctypes.CDLL(library_versions[-1][1])
 
 
+def _load_first_library(library_names, library_file_extensions,
+                        library_search_paths):
+    """
+    Finds, loads and returns the first found version of the library.
+    """
+    candidates = _find_library_candidates(
+        library_names,
+        library_file_extensions,
+        library_search_paths
+    )
+    library = None
+    for filename in candidates:
+        if os.path.isfile(filename):
+            try:
+                library = ctypes.CDLL(filename)
+                break
+            except OSError:
+                pass
+    if library is not None:
+        major_value = ctypes.c_int(0)
+        major = ctypes.pointer(major_value)
+        minor_value = ctypes.c_int(0)
+        minor = ctypes.pointer(minor_value)
+        rev_value = ctypes.c_int(0)
+        rev = ctypes.pointer(rev_value)
+        if hasattr(library, 'glfwGetVersion'):
+            library.glfwGetVersion(major, minor, rev)
+            version = (major_value.value, minor_value.value, rev_value.value)
+            if version >= (3, 0, 0):
+                return library
+    return None
+
+
 def _glfw_get_version(filename):
     """
     Queries and returns the library version tuple or None by using a
@@ -94,9 +129,9 @@ def _glfw_get_version(filename):
             else:
                 return None
 
-        try:
+        if sys.version_info[0] == 2:
             input_func = raw_input
-        except NameError:
+        else:
             input_func = input
         filename = input_func().strip()
 
@@ -140,15 +175,9 @@ def _get_library_search_paths():
         '/usr/lib/arm-linux-gnueabihf',
     ]
 
-    if sys.platform != 'darwin':
-        # manylinux2014 wheels contain libraries built for X11 and Wayland
-        if os.environ.get('PYGLFW_LIBRARY_VARIANT', '').lower() in ['wayland', 'x11']:
-            search_paths.insert(1, os.path.join(package_path, os.environ['PYGLFW_LIBRARY_VARIANT'].lower()))
-        elif os.environ.get('XDG_SESSION_TYPE') == 'wayland':
-            search_paths.insert(1, os.path.join(package_path, 'wayland'))
-        else:
-            # X11 is the default, even if XDG_SESSION_TYPE is not set
-            search_paths.insert(1, os.path.join(package_path, 'x11'))
+    package_path_variant = _get_package_path_variant(package_path)
+    if package_path_variant:
+        search_paths.insert(1, package_path_variant)
 
     if sys.platform == 'darwin':
         path_environment_variable = 'DYLD_LIBRARY_PATH'
@@ -159,6 +188,35 @@ def _get_library_search_paths():
     return search_paths
 
 
+def _get_frozen_library_search_paths():
+    """
+    Returns a list of library search paths for frozen executables.
+    """
+    current_path = os.path.abspath(os.getcwd())
+    executable_path = os.path.abspath(os.path.dirname(sys.executable))
+    package_path = os.path.abspath(os.path.dirname(__file__))
+    package_path_variant = _get_package_path_variant(package_path)
+    return [
+        executable_path,
+        package_path_variant,
+        package_path,
+        current_path
+    ]
+
+
+def _get_package_path_variant(package_path):
+    if sys.platform in ('darwin', 'win32'):
+        return None
+    # manylinux2014 wheels contain libraries built for X11 and Wayland
+    if os.environ.get('PYGLFW_LIBRARY_VARIANT', '').lower() in ['wayland', 'x11']:
+        return os.path.join(package_path, os.environ['PYGLFW_LIBRARY_VARIANT'].lower())
+    elif os.environ.get('XDG_SESSION_TYPE') == 'wayland':
+        return os.path.join(package_path, 'wayland')
+    else:
+        # X11 is the default, even if XDG_SESSION_TYPE is not set
+        return os.path.join(package_path, 'x11')
+
+
 if os.environ.get('PYGLFW_LIBRARY', ''):
     try:
         glfw = ctypes.CDLL(os.environ['PYGLFW_LIBRARY'])
@@ -192,6 +250,10 @@ elif sys.platform == 'win32':
             glfw = ctypes.CDLL(os.path.join(sys.prefix, 'Library', 'bin', 'glfw3.dll'))
         except OSError:
             pass
-else:
+elif not getattr(sys, "frozen", False):
     glfw = _load_library(['glfw', 'glfw3'], ['.so', '.dylib'],
                           _get_library_search_paths(), _glfw_get_version)
+else:
+
+    glfw = _load_first_library(['glfw', 'glfw3'], ['.so', '.dylib'],
+                               _get_frozen_library_search_paths())
